@@ -38,14 +38,8 @@
 
 #if defined(ALSA_BUILD) && LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
 #if defined(CONFIG_MODVERSIONS) && !defined(__GENKSYMS__) && !defined(__DEPEND__)
-#define MODVERSIONS
-#include <linux/modversions.h>
 #include "sndversions.h"
 #endif
-#ifdef SNDRV_NO_MODVERS
-#undef MODVERSIONS
-#undef _set_ver
-#endif
 #endif /* ALSA_BUILD && KERNEL < 2.6.0 */
 
 #include <linux/module.h>
@@ -182,6 +176,14 @@ static inline struct proc_dir_entry *PDE(const struct inode *inode)
 
 #endif /* < 2.6.0 */
 
+#include <linux/types.h>
+#ifndef __bitwise
+typedef __u16 __le16;
+typedef __u16 __be16;
+typedef __u32 __le32;
+typedef __u32 __be32;
+#endif
+
 #include <linux/wait.h>
 #ifndef wait_event_timeout
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
@@ -214,7 +216,7 @@ do {									\
 		if (!ret)						\
 			break;						\
 	}								\
-	current->state = TASK_RUNNING;					\
+	set_current_state(TASK_RUNNING);				\
 	remove_wait_queue(&wq, &__wait);				\
 } while (0)
 #endif /* 2.6.0 */
@@ -226,6 +228,59 @@ do {									\
 	__ret;								\
 })
 #endif
+#ifndef wait_event_interruptible_timeout
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
+#define __wait_event_interruptible_timeout(wq, condition, ret)		\
+do {									\
+	DEFINE_WAIT(__wait);						\
+									\
+	for (;;) {							\
+		prepare_to_wait(&wq, &__wait, TASK_INTERRUPTIBLE);	\
+		if (condition)						\
+			break;						\
+		if (!signal_pending(current)) {				\
+			ret = schedule_timeout(ret);			\
+			if (!ret)					\
+				break;					\
+			continue;					\
+		}							\
+		ret = -ERESTARTSYS;					\
+		break;							\
+	}								\
+	finish_wait(&wq, &__wait);					\
+} while (0)
+#else
+#define __wait_event_interruptible_timeout(wq, condition, ret)		\
+do {									\
+	wait_queue_t __wait;						\
+	init_waitqueue_entry(&__wait, current);				\
+									\
+	add_wait_queue(&wq, &__wait);					\
+	for (;;) {							\
+		set_current_state(TASK_INTERRUPTIBLE);			\
+		if (condition)						\
+			break;						\
+		if (!signal_pending(current)) {				\
+			ret = schedule_timeout(ret);			\
+			if (!ret)					\
+				break;					\
+			continue;					\
+		}							\
+		ret = -ERESTARTSYS;					\
+		break;							\
+	}								\
+	set_current_state(TASK_RUNNING);				\
+	remove_wait_queue(&wq, &__wait);				\
+} while (0)
+#endif /* 2.6.0 */
+#define wait_event_interruptible_timeout(wq, condition, timeout)	\
+({									\
+	long __ret = timeout;						\
+	if (!(condition))						\
+		__wait_event_interruptible_timeout(wq, condition, __ret); \
+	__ret;								\
+})
+#endif
 
 #ifndef CONFIG_HAVE_STRLCPY
 size_t snd_compat_strlcpy(char *dest, const char *src, size_t size);
@@ -235,10 +290,12 @@ size_t snd_compat_strlcat(char *dest, const char *src, size_t size);
 #endif
 
 #ifndef CONFIG_HAVE_SNPRINTF
-#include <stdarg.h>
 int snd_compat_snprintf(char * buf, size_t size, const char * fmt, ...);
-int snd_compat_vsnprintf(char *buf, size_t size, const char *fmt, va_list args);
 #define snprintf(buf,size,fmt,args...) snd_compat_snprintf(buf,size,fmt,##args)
+#endif
+#ifndef CONFIG_HAVE_VSNPRINTF
+#include <stdarg.h>
+int snd_compat_vsnprintf(char *buf, size_t size, const char *fmt, va_list args);
 #define vsnprintf(buf,size,fmt,args) snd_compat_vsnprintf(buf,size,fmt,args)
 #endif
 
@@ -303,6 +360,10 @@ typedef void irqreturn_t;
 #define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))
 #endif
 
+#ifndef BUG_ON
+#define BUG_ON(x) /* nothing */
+#endif
+
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
 #include <linux/devfs_fs_kernel.h>
 #ifdef CONFIG_DEVFS_FS
@@ -395,6 +456,7 @@ int snd_hack_usb_set_interface(struct usb_device *dev, int interface, int altern
 #define URB_ASYNC_UNLINK	USB_ASYNC_UNLINK
 #define usb_fill_int_urb	FILL_INT_URB
 #define usb_fill_bulk_urb	FILL_BULK_URB
+#define usb_fill_control_urb	FILL_CONTROL_URB
 #define usb_host_config		usb_config_descriptor
 #define usb_host_interface	usb_interface_descriptor
 #define usb_host_endpoint	usb_endpoint_descriptor
@@ -450,23 +512,40 @@ int snd_hack_usb_set_interface(struct usb_device *dev, int interface, int altern
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
 #include <linux/workqueue.h>
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 45) && !defined(__WORK_INITIALIZER)
+#define SND_WORKQUEUE_COMPAT
+struct workqueue_struct;
 struct work_struct {
+	unsigned long pending;
+	struct list_head entry;
 	void (*func)(void *);
 	void *data;
+	void *wq_data;
+	struct timer_list timer;
 };
 #define INIT_WORK(_work, _func, _data)			\
 	do {						\
 		(_work)->func = _func;			\
 		(_work)->data = _data;			\
+		init_timer(&(_work)->timer);		\
 	} while (0)
 #define __WORK_INITIALIZER(n, f, d) {			\
-	.func = (f),					\
-	.data = (d),					\
+		.func = (f),				\
+		.data = (d),				\
 	}
 #define DECLARE_WORK(n, f, d)				\
 	struct work_struct n = __WORK_INITIALIZER(n, f, d)
 int snd_compat_schedule_work(struct work_struct *work);
 #define schedule_work(w) snd_compat_schedule_work(w)
+struct workqueue_struct *snd_compat_create_workqueue(const char *name);
+#define create_workqueue(name) snd_compat_create_workqueue((name))
+void snd_compat_flush_workqueue(struct workqueue_struct *wq);
+#define flush_workqueue(wq) snd_compat_flush_workqueue((wq));
+void snd_compat_destroy_workqueue(struct workqueue_struct *wq);
+#define destroy_workqueue(wq) snd_compat_destroy_workqueue((wq));
+int snd_compat_queue_work(struct workqueue_struct *wq, struct work_struct *work);
+#define queue_work(wq, work) snd_compat_queue_work((wq), (work))
+int snd_compat_queue_delayed_work(struct workqueue_struct *wq, struct work_struct *work, unsigned long delay);
+#define queue_delayed_work(wq, work, delay) snd_compat_queue_delayed_work((wq), (work), (delay))
 #endif /* < 2.5.45 */
 #endif /* < 2.6.0 */
 
@@ -522,17 +601,26 @@ static inline void *snd_compat_vmap(struct page **pages, unsigned int count, uns
 
 /* sysfs */
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 2)
-struct class_simple;
-static inline void class_simple_device_add(struct class_simple *class, int devnum, ...) { return; }
-static inline void class_simple_device_remove(int devnum) { return; }
+struct class;
+static inline void class_device_create(struct class *class, int devnum, ...) { return; }
+static inline void class_device_destroy(struct class *class, int devnum) { return; }
 #endif
 
 /* msleep */
-#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0) && LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 6)) || (LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 29))
+#ifndef CONFIG_HAVE_MSLEEP
 void snd_compat_msleep(unsigned int msecs);
 #define msleep snd_compat_msleep
 #endif
 
+#ifndef CONFIG_HAVE_MSLEEP_INTERRUPTIBLE
+#include <linux/delay.h>
+unsigned long snd_compat_msleep_interruptible(unsigned int msecs);
+#define msleep_interruptible snd_compat_msleep_interruptible
+#ifndef ssleep
+#define ssleep(x) msleep((unsigned int)(x) * 1000)
+#endif
+#endif
+
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
 #define snd_card_set_dev(card,dev) /* no struct device */
 #endif
@@ -542,6 +630,9 @@ void snd_compat_msleep(unsigned int msecs);
 #define snd_dma_isa_data()	NULL
 #define snd_dma_sbus_data(sbus)	((struct device *)(sbus))
 #define snd_dma_continuous_data(x)	((struct device *)(unsigned long)(x))
+#ifndef DMA_32BIT_MASK
+#define DMA_32BIT_MASK 0xffffffff
+#endif
 #endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
@@ -577,9 +668,11 @@ void snd_compat_msleep(unsigned int msecs);
 
 /* dump_stack hack */
 #ifndef CONFIG_HAVE_DUMP_STACK
+#undef dump_stack
 #define dump_stack()
 #endif
 
+#ifdef CONFIG_PCI
 #ifndef CONFIG_HAVE_PCI_DEV_PRESENT
 #include <linux/pci.h>
 #ifndef PCI_DEVICE
@@ -590,6 +683,7 @@ void snd_compat_msleep(unsigned int msecs);
 int snd_pci_dev_present(const struct pci_device_id *ids);
 #define pci_dev_present(x) snd_pci_dev_present(x)
 #endif
+#endif
 
 /* power management compatibility layer */
 #ifdef CONFIG_PM
@@ -609,5 +703,102 @@ void *snd_compat_kcalloc(size_t n, size_t size, int gfp_flags);
 #endif
 #endif
 
+/* DEFINE_SPIN/RWLOCK (up to 2.6.11-rc2) */
+#if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 11)
+#include <linux/spinlock.h>
+#ifndef DEFINE_SPINLOCK
+#define DEFINE_SPINLOCK(x) spinlock_t x = SPIN_LOCK_UNLOCKED
+#define DEFINE_RWLOCK(x) rwlock_t x = RW_LOCK_UNLOCKED
+#endif
+#endif
+
+/* pm_message_t type */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 11)
+#include <linux/pm.h>
+#ifndef PMSG_FREEZE
+typedef u32 /*__bitwise*/ pm_message_t;
+#define PMSG_FREEZE	3
+#define PMSG_SUSPEND	3
+#define PMSG_ON		0
+#endif
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 11)
+#ifdef CONFIG_PCI
+#include <linux/pci.h>
+#ifndef PCI_D0
+#define PCI_D0     0
+#define PCI_D1     1
+#define PCI_D2     2
+#define PCI_D3hot  3
+#define PCI_D3cold 4
+#endif
+#endif
+#endif
+
+/* vprintk */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 9)
+#include <linux/kernel.h>
+static inline void snd_compat_vprintk(const char *fmt, va_list args)
+{
+	char tmpbuf[512];
+	vsnprintf(tmpbuf, sizeof(tmpbuf), fmt, args);
+	printk(tmpbuf);
+}
+#define vprintk snd_compat_vprintk
+#endif
+
+#if defined(CONFIG_GAMEPORT) || defined(CONFIG_GAMEPORT_MODULE)
+#define wait_ms gameport_wait_ms
+#include <linux/gameport.h>
+#undef wait_ms
+#ifndef to_gameport_driver
+#include <linux/slab.h>
+/* old gameport interface */
+struct snd_gameport {
+	struct gameport gp;
+	void *port_data;
+};
+static inline struct gameport *gameport_allocate_port(void)
+{
+	extern void *snd_hidden_kcalloc(size_t n, size_t size, int flags);
+	struct snd_gameport *gp;
+#ifdef CONFIG_SND_DEBUG_MEMORY
+	gp = snd_hidden_kcalloc(1, sizeof(*gp), GFP_KERNEL);
+#else
+	gp = kcalloc(1, sizeof(*gp), GFP_KERNEL);
+#endif
+	if (gp)
+		return &gp->gp;
+	return NULL;
+}
+static inline void snd_gameport_unregister_port(struct gameport *gp)
+{
+	extern void snd_hidden_kfree(const void *obj);
+	gameport_unregister_port(gp);
+#ifdef CONFIG_SND_DEBUG_MEMORY
+	snd_hidden_kfree(gp);
+#else
+	kfree(gp);
+#endif
+}
+#undef gameport_unregister_port
+#define gameport_unregister_port(gp)	snd_gameport_unregister_port(gp)
+#define gameport_set_port_data(gp,r) (((struct snd_gameport *)(gp))->port_data = (r))
+#define gameport_get_port_data(gp) ((struct snd_gameport *)(gp))->port_data
+#define gameport_set_dev_parent(gp,xdev)
+#define gameport_set_name(gp,x)
+#define gameport_set_phys(gp,x,y)
+#endif /* to_gameport_driver */
+#endif /* GAMEPORT || GAMEPORT_MODULE */
+
+/* use pci_module_init on 2.4 kernels */
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 4, 0) && LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
+#ifdef CONFIG_PCI
+#include <linux/pci.h>
+#undef pci_register_driver
+#define pci_register_driver	pci_module_init
+#endif
+#endif
 
 #endif /* __SOUND_LOCAL_DRIVER_H */
