@@ -183,6 +183,14 @@ static const struct gtype##_id * __module_##gtype##_table \
 #define list_for_each(pos, head) \
 	for (pos = (head)->next; pos != (head); pos = pos->next)
 
+/**
+ * list_for_each_prev	-	iterate over a list backwards
+ * @pos:	the &struct list_head to use as a loop counter.
+ * @head:	the head for your list.
+ */
+#define list_for_each_prev(pos, head) \
+	for (pos = (head)->prev; pos != (head); pos = pos->prev)
+
 #ifndef IORESOURCE_IO
 struct resource {
 	const char *name;
@@ -232,6 +240,7 @@ void snd_pci_compat_free_consistent(struct pci_dev *, long, void *, dma_addr_t);
 #define pci_register_driver snd_pci_compat_register_driver
 #define pci_unregister_driver snd_pci_compat_unregister_driver
 #define pci_set_power_state snd_pci_compat_set_power_state
+#define pci_match_device snd_pci_compat_match_device
 
 #define pci_dma_supported snd_pci_compat_dma_supported
 
@@ -311,15 +320,7 @@ void snd_pci_compat_release_regions(struct pci_dev *pdev);
 void snd_pci_compat_save_state(struct pci_dev *pdev, u32 *buf);
 void snd_pci_compat_restore_state(struct pci_dev *pdev, u32 *buf);
 
-static inline int pci_module_init(struct pci_driver *drv)
-{
-	int res = snd_pci_compat_register_driver(drv);
-	if (res < 0)
-		return res;
-	if (res == 0)
-		return -ENODEV;
-	return 0;
-}
+#define pci_module_init	snd_pci_compat_register_driver
 
 #endif /* CONFIG_PCI */
 
@@ -470,6 +471,14 @@ extern inline int pm_send(struct pm_dev *dev, pm_request_t rqst, void *data)
 
 #include <linux/usb.h>
 
+struct usb_ctrlrequest {
+	__u8 bRequestType;
+	__u8 bRequest;
+	__u16 wValue;
+	__u16 wIndex;
+	__u16 wLength;
+} __attribute__ ((packed));
+
 struct snd_compat_usb_device_id {
 	__u16		match_flags;
 	__u16		idVendor;
