@@ -41,7 +41,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * msnd_pinnacle.c,v 1.11 2004/07/01 08:38:28 tiwai Exp
+ * msnd_pinnacle.c,v 1.12 2005/01/31 16:07:20 cladisch Exp
  *
  * 12-3-2000  Modified IO port validation  Steve Sycamore
  *
@@ -555,11 +555,10 @@ static int __init snd_msnd_probe(void)
 	char *pinfiji = "Pinnacle/Fiji";
 #endif
 
-	if (check_region(dev.io, dev.numio)) {
+	if (!request_region(dev.io, dev.numio, "probing")) {
 		printk(KERN_ERR LOGNAME ": I/O port conflict\n");
 		return -ENODEV;
 	}
-	request_region(dev.io, dev.numio, "probing");
 
 	if ( snd_msnd_reset_dsp() < 0) {
 		release_region(dev.io, dev.numio);
@@ -1761,12 +1760,11 @@ static int __init snd_msnd_init(void)
 		/* Joystick */
 		pinnacle_devs[3].io0 = joystick_io;
 
-		if (check_region(cfg, 2)) {
+		if (!request_region(cfg, 2, "Pinnacle/Fiji Config")) {
 			printk(KERN_ERR LOGNAME ": Config port 0x%x conflict\n", cfg);
 			return -EIO;
 		}
 
-		request_region(cfg, 2, "Pinnacle/Fiji Config");
 		if (snd_msnd_pinnacle_cfg_devices(cfg, reset, pinnacle_devs)) {
 			printk(KERN_ERR LOGNAME ": Device configuration error\n");
 			release_region(cfg, 2);
