@@ -62,6 +62,7 @@ static int init_hw(echoaudio_t *chip, u16 device_id, u16 subdevice_id)
 					ECHOCAPS_HAS_DIGITAL_MODE_ADAT;
 	chip->digital_mode =		DIGITAL_MODE_SPDIF_RCA;
 	chip->professional_spdif = FALSE;
+	chip->digital_in_automute = TRUE;
 
 	/* Load the DSP and the ASIC on the PCI card */
 	if ((err = load_firmware(chip)) < 0)
@@ -74,11 +75,11 @@ static int init_hw(echoaudio_t *chip, u16 device_id, u16 subdevice_id)
 		return err;
 
 	/* Set professional nominal levels (FALSE is +4dBu) */
-	for (i = 0; i < NUM_ANALOG_BUSSES_OUT; i++)
+	for (i = 0; i < num_analog_busses_out(chip); i++)
 		err = set_nominal_level(chip, i, FALSE);
 
-	for (i = 0; i < NUM_ANALOG_BUSSES_IN; i++)
-		err = set_nominal_level(chip, BX_ANALOG_IN + i, FALSE);
+	for (i = 0; i < num_analog_busses_in(chip); i++)
+		err = set_nominal_level(chip, bx_analog_in(chip) + i, FALSE);
 
 	/* Set the digital mode to S/PDIF RCA */
 	set_digital_mode(chip, DIGITAL_MODE_SPDIF_RCA);
@@ -345,20 +346,26 @@ static int set_input_clock(echoaudio_t *chip, u16 clock)
 
 static int switch_asic(echoaudio_t *chip, const struct firmware *asic)
 {
-	s8 monitors[MONITOR_ARRAY_SIZE];
+	s8 *monitors;
 
 	/*  Check to see if this is already loaded */
 	if (asic != chip->asic_code) {
+		monitors = kmalloc(MONITOR_ARRAY_SIZE, GFP_KERNEL);
+		if (! monitors)
+			return -ENOMEM;
+
 		memcpy(monitors, chip->comm_page->monitors, MONITOR_ARRAY_SIZE);
 		memset(chip->comm_page->monitors, ECHOGAIN_MUTED, MONITOR_ARRAY_SIZE);
 
 		/* Load the desired ASIC */
 		if (load_asic_generic(chip, DSP_FNC_LOAD_LAYLA24_EXTERNAL_ASIC, asic) < 0) {
 			memcpy(chip->comm_page->monitors, monitors, MONITOR_ARRAY_SIZE);
+			kfree(monitors);
 			return -EIO;
 		}
 		chip->asic_code = asic;
 		memcpy(chip->comm_page->monitors, monitors, MONITOR_ARRAY_SIZE);
+		kfree(monitors);
 	}
 
 	return 0;
