@@ -33,8 +33,19 @@ struct pnp_card {
 
 struct pnp_dev {
 	struct isapnp_dev p;
+	void * driver_data;
 };
 
+static inline void *pnp_get_drvdata (struct pnp_dev *dev)
+{
+	return dev->driver_data;
+}
+
+static inline void pnp_set_drvdata (struct pnp_dev *dev, void *data)
+{
+	dev->driver_data = data;
+}
+
 /*
  * Resource Management
  */
@@ -198,6 +209,7 @@ void pnp_init_resource_table(struct pnp_resource_table *table);
 int pnp_manual_config_dev(struct pnp_dev *dev, struct pnp_resource_table *res, int mode);
 int pnp_activate_dev(struct pnp_dev *dev);
 static inline int pnp_is_active(struct pnp_dev *dev) { return dev->p.active; }
+int pnp_disable_dev(struct pnp_dev *dev);
 
 #else
 
@@ -211,6 +223,7 @@ static inline void pnp_init_resource_table(struct pnp_resource_table *table) { ;
 static inline int pnp_manual_config_dev(struct pnp_dev *dev, struct pnp_resource_table *res, int mode) { return -ENODEV; }
 static inline int pnp_activate_dev(struct pnp_dev *dev) { return -ENODEV; }
 static inline int pnp_is_active(struct pnp_dev * dev) { return -ENODEV; }
+static inline int pnp_disable_dev(struct pnp_dev *dev) { return -ENODEV; }
 
 #endif
 
