@@ -102,6 +102,7 @@ static char *kernel_deps[] = {
 	"PCI",
 	"SBUS",
 	"I2C",
+	"INPUT",
 	"L3",
 	"USB",
 	"PCMCIA",
@@ -112,48 +113,57 @@ static char *kernel_deps[] = {
 	"SPARC64",
 	"PPC",
 	"PPC64",
+	"PPC_PMAC",
 	"X86_64",
 	"X86",
+	"MIPS",
 	"MIPS64",
 	"IA32_EMULATION",
 	/* architecture specific */
 	"ARCH_SA1100",
+	"ARCH_PXA",
 	"X86_PC9800",
 	/* other drivers */
 	"RTC",
 	"GAMEPORT",
 	"VIDEO_DEV",
+	"FW_LOADER",
+	"ASIHPK", /* external tree */
 	/* some flags */
 	"EXPERIMENTAL",
 	NULL
 };
 
 /* % -> always true */
+/* # -> define */
 static char *no_cards[] = {
 	"%SOUND",
 	"SOUND_PRIME",
 	"%SND",
-	"SND_TIMER",
-	"SND_HWDEP",
-	"SND_RAWMIDI",
-	"SND_PCM",
+	"#SND_TIMER",
+	"#SND_HWDEP",
+	"#SND_RAWMIDI",
+	"#SND_PCM",
 	"SND_SEQUENCER",
 	"SND_MIXER_OSS",
 	"SND_PCM_OSS",
 	"SND_SEQUENCER_OSS",
 	"SND_OSSEMUL",
 	"SND_RTCTIMER",
+	"SND_HPET",
 	"SND_DEBUG",
 	"SND_DEBUG_MEMORY",
 	"SND_DEBUG_DETECT",
 	"SND_VERBOSE_PRINTK",
 	"SND_BIT32_EMUL",
-	"SND_OPL3_LIB",
-	"SND_OPL4_LIB",
-	"SND_VX_LIB",
-	"SND_MPU401_UART",
-	"SND_GUS_SYNTH",
-	"SND_AC97_CODEC",
+	"#SND_OPL3_LIB",
+	"#SND_OPL4_LIB",
+	"#SND_VX_LIB",
+	"#SND_MPU401_UART",
+	"#SND_GUS_SYNTH",
+	"#SND_AC97_CODEC",
+	"#SND_PXA2XX_PCM",
+	"#SND_GENERIC_PM",
 	NULL
 };
 
@@ -285,9 +295,9 @@ static int read_file_1(const char *filename, struct cond **template)
 		}
 		switch (state) {
 		case READ_STATE_CONFIG:
-			if (!strncmp(buffer, "\ttristate ", 10))
+			if (!strncmp(buffer, "\ttristate", 9))
 				dep->is_bool = 0;
-			else if (!strncmp(buffer, "\tbool ", 6))
+			else if (!strncmp(buffer, "\tbool", 5))
 				dep->is_bool = 1;
 			else if (!strncmp(buffer, "\tdepends on ", 12))
 				add_dep(dep, buffer + 12, *template);
@@ -765,7 +775,7 @@ static int is_toplevel(struct dep *dep)
 	}
 	for (idx = 0; no_cards[idx]; idx++) {
 		str = no_cards[idx];
-		if (*str == '%')
+		if (*str == '%' || *str == '#')
 			str++;
 		if (!strcmp(str, dep->name))
 			return 0;
@@ -773,6 +783,33 @@ static int is_toplevel(struct dep *dep)
 	return 1;
 }
 
+// whether to output AC_DEFINE() for this variable
+static int output_ac_define(struct dep *dep)
+{
+	int idx;
+	char *str;
+	
+	if (dep == NULL)
+		return 0;
+	for (idx = 0; kernel_deps[idx]; idx++) {
+		if (!strcmp(kernel_deps[idx], dep->name))
+			return 0;
+	}
+	for (idx = 0; no_cards[idx]; idx++) {
+		int def = 0;
+		str = no_cards[idx];
+		if (*str == '%')
+			str++;
+		if (*str == '#') {
+			def = 1;
+			str++;
+		}
+		if (!strcmp(str, dep->name))
+			return def;
+	}
+	return 1;
+}
+
 // is CONFIG_ variable is always true
 static int is_always_true(struct dep *dep)
 {
@@ -898,20 +935,16 @@ static void output_acinclude(void)
 	       "  [                        cards may be separated with commas; ]\n"
 	       "  [                        'all' compiles all drivers; ]\n"
 	       "  [                        Possible cards are: ]\n");
-	output_card_list(all_deps, 24, 50, 1);
-	printf(" ]\n");
-	printf("  [                        Possible additional options are: ]\n");
-	output_card_list(all_deps, 24, 50, 0);
+	output_card_list(all_deps, 26, 50, 1);
 	printf(" ],\n");
 	printf("  cards=\"$withval\", cards=\"all\")\n");
-
+	printf("SELECTED_CARDS=`echo $cards | sed 's/,/ /g'`\n");
 	/* check cards */
-	printf("cards=`echo $cards | sed 's/,/ /g'`\n");
-	printf("for card in $cards; do\n"
+	printf("for card in $SELECTED_CARDS; do\n"
 	       "  probed=\n");
 	for (tempdep = all_deps; tempdep; tempdep = tempdep->next) {
 		int put_if;
-		if (!is_toplevel(tempdep))
+		if (!is_toplevel(tempdep) || tempdep->is_bool)
 			continue;
 		text = get_card_name(tempdep->name);
 		if (! text)
@@ -929,10 +962,10 @@ static void output_acinclude(void)
 			if (cond->not)
 				printf(" ! ");
 			for (j = 0; j < cond->left; j++)
-				printf("(");
-			printf("( test \"$CONFIG_%s\" == \"y\" -o \"$CONFIG_%s\" == \"m\" )", cond->name, cond->name);
+				printf("( ");
+			printf("( test \"$CONFIG_%s\" = \"y\" -o \"$CONFIG_%s\" = \"m\" )", cond->name, cond->name);
 			for (j = 0; j < cond->right; j++)
-				printf(")");
+				printf(" )");
 			put_if = 1;
 			cond_prev = cond;
 		}
@@ -964,10 +997,79 @@ static void output_acinclude(void)
 	       "    AC_MSG_ERROR(Unsupported soundcard $card)\n"
 	       "  fi\n"
 	       "done\n\n");
-	printf("AC_MSG_RESULT($cards)\n\n");
+	/* options */
+	printf("AC_ARG_WITH(card_options,\n"
+	       "  [  --with-card-options=<list> enable driver options in <list>; ]\n"
+	       "  [                        options may be separated with commas; ]\n"
+	       "  [                        'all' enables all options; ]\n"
+	       "  [                        Possible options are: ]\n");
+	output_card_list(all_deps, 26, 50, 0);
+	printf(" ],\n");
+	printf("  cards=\"$withval\", cards=\"all\")\n");
+	printf("SELECTED_OPTIONS=`echo $cards | sed 's/,/ /g'`\n");
+	/* check cards */
+	printf("for card in $SELECTED_OPTIONS; do\n"
+	       "  probed=\n");
+	for (tempdep = all_deps; tempdep; tempdep = tempdep->next) {
+		int put_if;
+		if (!is_toplevel(tempdep) || !tempdep->is_bool)
+			continue;
+		text = get_card_name(tempdep->name);
+		if (! text)
+			continue;
+		printf("  if test \"$card\" = \"all\" -o \"$card\" = \"%s\"; then\n", text);
+		free(text);
+		put_if = 0;
+		for (cond = tempdep->cond, cond_prev = NULL; cond; cond = cond->next) {
+			if (!put_if)
+				printf("    if ");
+			else {
+				printf(cond_prev->type == COND_AND ? " &&" : " ||");
+				printf("\n      ");
+			}
+			if (cond->not)
+				printf(" ! ");
+			for (j = 0; j < cond->left; j++)
+				printf("( ");
+			printf("( test \"$CONFIG_%s\" = \"y\" -o \"$CONFIG_%s\" = \"m\" )", cond->name, cond->name);
+			for (j = 0; j < cond->right; j++)
+				printf(" )");
+			put_if = 1;
+			cond_prev = cond;
+		}
+		if (put_if)
+			printf("; then\n");
+
+		sel_remove_hitflags();
+		for (sel = tempdep->sel; sel; sel = sel->next)
+			sel_print_acinclude(sel);			
+		for (sel = tempdep->sel; sel; sel = sel->next) {
+			if (is_always_true(sel->dep))
+				continue;
+			printf("      CONFIG_%s=\"%c\"\n", sel->name,
+			       (sel->dep && sel->dep->is_bool) ? 'y' : 'm');
+		}
+		text = convert_to_config_uppercase("CONFIG_", tempdep->name);
+		printf("      %s=\"y\"\n", text);
+		free(text);
+		printf("      probed=1\n");
+		if (put_if)
+			printf("    elif test -z \"$probed\"; then\n"
+			       "      probed=0\n"
+			       "    fi\n");
+		printf("  fi\n");
+	}
+	printf("  if test -z \"$probed\"; then\n"
+	       "    AC_MSG_ERROR(Unknown option $card)\n"
+	       "  elif test \"$probed\" = \"0\" -a \"$card\" != \"all\"; then\n"
+	       "    AC_MSG_ERROR(Unsupported option $card)\n"
+	       "  fi\n"
+	       "done\n\n");
+
+	printf("AC_MSG_RESULT($SELECTED_CARDS)\n\n");
 	printf("CONFIG_SND=\"m\"\n");
 	for (tempdep = all_deps; tempdep; tempdep = tempdep->next) {
-		if (!is_toplevel(tempdep))
+		if (!output_ac_define(tempdep))
 			continue;
 		text = convert_to_config_uppercase("CONFIG_", tempdep->name);
 		printf("if test -n \"$%s\"; then\n", text);
